import { app } from '@/app.ts';
import { genOutTradeNo, genSubject } from './utils/index.ts';
import './wx-pay.ts';
import './pay.ts';
import { TradeModel } from '../trade/model.ts';
import { log } from '@/logger/index.ts';
import { getLoginUser } from '@/modules/user.ts';

app
  .route({
    path: 'vip',
    key: 'pay',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { level, money, payMode = 'alipay', out_trade_no, month = 1 } = ctx.query.data || {};
    const userId = tokenUser?.id;
    // 微信支付在新页面，所以，这个借口可能没有userId
    if (!level) {
      ctx.throw(400, 'level is required');
    }
    if (isNaN(Number(money))) {
      ctx.throw(400, 'money must be a number');
    }
    const _out_trade_no = out_trade_no || genOutTradeNo();
    const subject = genSubject(level);
    let result;
    if (payMode === 'alipay') {
      const callResult = await app.call({
        path: 'alipay',
        key: 'pay',
        payload: {
          data: {
            money,
            subject,
            out_trade_no: _out_trade_no,
          },
        },
      });
      if (callResult.code === 200) {
        result = callResult.body;
      } else {
        log.error('pay failed', {
          message: callResult?.message,
          body: callResult?.body,
        });
        ctx.throw(400, callResult?.message || 'pay failed');
      }
    } else if (payMode === 'wx') {
      const callResult = await app.call({
        path: 'wx',
        key: 'pay',
        payload: {
          data: {
            money,
            subject,
            out_trade_no: _out_trade_no,
          },
        },
      });
      if (callResult.code === 200) {
        result = callResult.body;
      } else {
        log.error('pay failed', {
          message: callResult?.message,
          body: callResult?.body,
        });
        ctx.throw(400, callResult?.message || 'pay failed');
      }
    } else {
      log.error('payMode is Unsupported', {
        payMode,
      });
      ctx.throw(400, `current payMode ${payMode} is Unsupported`);
    }
    // 检查是否存在订单
    let trade: TradeModel;
    if (out_trade_no) {
      trade = await TradeModel.findOne({
        where: {
          out_trade_no: out_trade_no,
        },
      });
      if (trade && trade.uid !== userId) {
        ctx.throw(403, 'trade has exist but not permission');
      }
    }
    if (!trade) {
      trade = await TradeModel.create({
        out_trade_no: _out_trade_no,
        money: money,
        subject: subject,
        uid: userId,
        data: {
          list: [],
          target: {
            type: 'vip' as const,
            level: level,
            month: month, // 月数
            updateTime: new Date(),
            category: 'center' as const,
          },
        },
      });
    } else {
      const newTarget = {
        type: 'vip' as const,
        level: level,
        month: month, // 月数
        updateTime: new Date(),
        category: 'center' as const,
      };
      trade.data = {
        ...trade.data,
        target: newTarget,
      };
      await trade.save({ fields: ['data'] });
    }
    log.info(`[pay] ${userId} pay ${subject} ${money} ${_out_trade_no} ${trade.id}`);
    ctx.body = result;
  })
  .addTo(app);
