import { log } from '@/logger/index.ts';
import { payWechatConfig } from '@/modules/pay-wx-config.ts';
import WxPay from '@kevisual/wechatpay'; // 支持使用require

// tsx src/routes/pay/wechatpay/init.ts
export const initSdk = () => {
  const pay = new WxPay(payWechatConfig);
  const sign = (params: any, url = '/v3/pay/transactions/native') => {
    const nonce_str = Math.random().toString(36).substr(2, 15), // 随机字符串
      timestamp = parseInt(+new Date() / 1000 + '').toString(); // 时间戳 秒
    // url = '/v3/pay/transactions/h5'

    // # 获取签名
    const signature = pay.getSignature('POST', nonce_str, timestamp, url, params); //# 如果是get 请求 则不需要params 参数拼接在url上 例如 /v3/pay/transactions/id/12177525012014?mchid=1230000109
    // # 获取头部authorization 参数
    const authorization = pay.getAuthorization(nonce_str, timestamp, signature);
    return {
      nonce_str,
      timestamp,
      authorization,
    };
  };
  const createParams = (amount: number, orderId: string, opts?: any) => {
    return {
      appid: payWechatConfig.appid,
      mchid: payWechatConfig.mchid,
      description: 'Image形象店-深圳腾大-QQ公仔',
      out_trade_no: orderId,
      notify_url: 'https://www.weixin.qq.com/wxpay/pay.php',
      ...opts,
      amount: {
        total: amount,
      },
    };
  };

  return {
    pay,
    createParams,
    sign,
  };
};
export type Transactions = {
  code_url: string;
};

export const getPaymentQRCode = async ({ amount, out_trade_no, subject }: { amount: number; out_trade_no: string; subject: string }, opts?: any) => {
  try {
    const sdk = initSdk();
    const result = await sdk.pay.transactions_native({
      description: subject,
      out_trade_no: out_trade_no,
      notify_url: 'https://kevisual.xiongxiao.me/api/s1/pay/notify/wx',
      amount: {
        total: amount,
      },
      time_expire: new Date(Date.now() + 1000 * 60 * 60 * 10).toISOString(),
      ...opts,
    });
    const data: Transactions = result.data;
    if (!data.code_url) {
      throw new Error('code_url is generate failed');
    }
    return {
      url: data.code_url,
    };
  } catch (error) {
    console.error('Error fetching payment QR code:', error);
    throw error;
  }
};
export const verify = async (params, body) => {
  const sdk = initSdk();
  const result = await sdk.pay.verifySign(params);
  return result;
};
export const verifySignature = async (req: any, body: any) => {
  const sdk = initSdk();
  const headers = req.headers;
  const params = {
    apiSecret: payWechatConfig.key,
    body: body, // 请求体 body
    signature: headers['wechatpay-signature'],
    serial: headers['wechatpay-serial'],
    nonce: headers['wechatpay-nonce'],
    timestamp: headers['wechatpay-timestamp'],
  };
  const result = await sdk.pay.verifySign(params);
  // 拉取平台证书失败
  let decryptBody = {};
  const { ciphertext, associated_data, nonce, key } = body?.resource;
  decryptBody = sdk.pay.decipher_gcm(ciphertext, associated_data, nonce, key);
  if (!result) {
    log.warn('verifySignature error', {
      body,
      result,
      params,
    });
  }
  // 无脑返回true
  return { result: result, decryptBody };
};
export const decryptBody = async (body: any) => {
  const sdk = initSdk();
  const { ciphertext, associated_data, nonce, key } = body?.resource;
  return sdk.pay.decipher_gcm(ciphertext, associated_data, nonce, key);
};
