import { payWechatConfig } from '@/modules/pay-wx-config.ts';
import crypto from 'crypto';

/**
 * 使用AES-GCM算法解密数据（Node.js环境）
 * @param {string} nonce - 加密使用的nonce
 * @param {string} ciphertext - Base64编码的密文
 * @param {string} associatedData - 关联数据
 * @returns {string} - 解密后的明文
 */
export const decrypt = (nonce: string, ciphertext: string, associatedData: string): string => {
  // 密钥
  const key = payWechatConfig.key;
  
  // 创建解密器
  const decipher = crypto.createDecipheriv(
    'aes-256-gcm',
    Buffer.from(key),
    Buffer.from(nonce)
  );
  
  // 设置额外的认证数据
  decipher.setAAD(Buffer.from(associatedData));
  
  // Base64解码密文
  const ciphertextBuffer = Buffer.from(ciphertext, 'base64');
  
  // 提取认证标签（GCM模式的认证标签通常是密文的最后16字节）
  const authTag = ciphertextBuffer.slice(ciphertextBuffer.length - 16);
  const actualCiphertext = ciphertextBuffer.slice(0, ciphertextBuffer.length - 16);
  
  // 设置认证标签
  decipher.setAuthTag(authTag);
  
  // 解密
  let decrypted = decipher.update(actualCiphertext);
  decrypted = Buffer.concat([decrypted, decipher.final()]);
  
  // 返回解密后的字符串
  return decrypted.toString();
};
