import { app } from '@/app.ts';
import { TradeModel } from './model.ts';
import { UserModel } from '@/modules/user.ts';
// 获取用户的交易列表
app
  .route({
    path: 'trade',
    key: 'list',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { page = 1, pageSize = 10 } = ctx.query.data || {};
    const trades = await TradeModel.findAndCountAll({
      where: {
        // uid: tokenUser.id,
      },
      order: [['createdAt', 'DESC']],
      offset: (page - 1) * pageSize,
      limit: pageSize,
    });
    const uidList = trades.rows.map((trade) => trade.uid);
    const userList = await UserModel.findAll({
      where: {
        id: uidList,
      },
      attributes: ['id', 'username', 'nickname'],
    });

    ctx.body = {
      list: trades.rows,
      userList,
      pagination: {
        page,
        current: page,
        pageSize,
        total: trades.count,
      },
    };
  })
  .addTo(app);

// 根据订单号查找或创建交易记录
app
  .route({
    path: 'trade',
    key: 'get',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const { out_trade_no, id } = ctx.query.data || {};
    const tokenUser = ctx.state.tokenUser;

    if (!out_trade_no && !id) {
      ctx.throw(400, 'out_trade_no or id is required');
    }
    let trade: TradeModel | null = null;
    if (out_trade_no) {
      trade = await TradeModel.findOne({
        where: {
          out_trade_no,
        },
      });
    } else {
      trade = await TradeModel.findByPk(id);
    }
    if (!trade) {
      ctx.throw(404, 'trade not found');
    }

    ctx.body = trade;
  })
  .addTo(app);
