import { app } from '@/app.ts';
import { log } from '@/logger/index.ts';
import { VipModel } from '../vip/models/index.ts';
import { TradeModel } from './model.ts';

app
  .route({
    path: 'trade',
    key: 'paySuccess',
  })
  .define(async (ctx) => {
    const { trade } = ctx;
    if (!trade) {
      log.error('not has trade info');
      ctx.throw('not has trade info');
    }
    const { uid, data } = trade as TradeModel;
    const { target } = data;
    if (target.type === 'vip') {
      const { level, month } = target;
      // 更新的结束时间， 添加一个月。
      const vip = await VipModel.findOne({
        where: {
          userId: uid,
          category: target.category,
        },
      });
      if (!vip) {
        log.warn('not has vip info', {
          uid,
          tradeId: trade.id,
          outTradeNo: trade.out_trade_no,
          money: trade.money,
          category: target.category,
        });
        return;
      }
      vip?.addVip({
        level,
        month,
      });
    }
  })
  .addTo(app);
